@@ -1,25 +1,14 @@
 """MESSENGER UVVS data class"""
+import mathMB
 import numpy as np
 import pandas as pd
 import copy
 from astropy import units as u
 
-from nexoclom import Input, Output, LOSResult
+from nexoclom import Input, LOSResult
+import mathMB
 from .database_setup import database_connect
 from .plot_methods import plot_bokeh, plot_plotly, plot_fitted
-from mathMB import fit_model
-
-
-def format_query(comparison):
-    '''Try to impose some regularity on the query.'''
-    query = comparison.lower()
-    
-    # Remove some extra spaces
-    chars = ['=', '>', '<', '!=', '>=', '<=']
-    for char in chars:
-        query = query.replace(f' {char} ', char)
-        
-    return query
 
 
 class InputError(Exception):
@@ -166,9 +155,9 @@ class MESSENGERdata:
     ::
     
         >>> inputs = Input('Ca.spot.Maxwellian.input')
-        >>> CaData.model(inputs, 1e5, label='Model 1')
+        >>> CaData.simulate_data_from_inputs(inputs, 1e5, label='Model 1')
         >>> inputs.speeddist.temperature /= 2.  # Run model with different temperature
-        >>> CaData.model(inputs, 1e5, label='Model 2')
+        >>> CaData.simulate_data_from_inputs(inputs, 1e5, label='Model 2')
         
     4. Plotting data
     
@@ -185,21 +174,21 @@ class MESSENGERdata:
 
     
     """
-    def __init__(self, species=None, comparisons=None, load_spectra=False):
+    def __init__(self, species, comparisons=None, load_spectra=False):
         allspecies = ['Na', 'Ca', 'Mg']
-        self.species = None
+        if species in allspecies:
+            self.species = species
+        else:
+            raise InputError('MESSENGERdata.__init__', f'{species} not valid.')
+        
         self.frame = None
         self.query = None
         self.data = None
         self.taa = None
-        self.app = None
+        self.app = None  # Plotly app
         self.model_info = {}
-        if species is None:
-            pass
-        elif species not in allspecies:
-            # Return list of valid species
-            print(f"Valid species are {', '.join(allspecies)}")
-        elif comparisons is None:
+        
+        if comparisons is None:
             # Return list of queryable fields
             with database_connect() as con:
                 columns = pd.read_sql(
@@ -211,20 +200,21 @@ class MESSENGERdata:
         else:
             # Run the query and try to make the object
             query = f'''SELECT * from {species}uvvsdata, {species}pointing
-                        WHERE unum=pnum and {comparisons}
+                        WHERE unum=pnum and ({comparisons})
                         ORDER BY unum'''
             try:
                 with database_connect() as con:
                     data = pd.read_sql(query, con)
             except Exception:
+                print('Requested data:')
+                print(query)
                 raise InputError('MESSENGERdata.__init__',
                                  'Problem with comparisons given.')
 
             if len(data) > 0:
-                self.species = species
                 self.frame = data.frame[0]
-                self.query = format_query(comparisons)
-                self.taa = np.median(data.taa)
+                self.query = self._format_query(comparisons)
+                self.taa = np.median(data.taa)*u.rad
                 
                 data.drop(['species', 'frame'], inplace=True, axis=1)
                 data.loc[data.alttan < 0, 'alttan'] = 1e10
@@ -254,11 +244,7 @@ class MESSENGERdata:
                 print('No data found')
                 
     def __str__(self):
-        result = (f'Species: {self.species}\n'
-                  f'Query: {self.query}\n'
-                  f'Frame: {self.frame}\n'
-                  f'Object contains {len(self)} spectra.')
-        return result
+        return self.__repr__()
 
     def __repr__(self):
         result = ('MESSENGER UVVS Data Object\n'
@@ -281,10 +267,8 @@ class MESSENGERdata:
         else:
             raise TypeError
 
-        new = MESSENGERdata()
-        new.species = self.species
-        new.frame = self.frame
-        new.query = self.query
+        new = copy.deepcopy(self)
+        new.query = self.query + ' [Subset]'
         new.taa = self.taa
         new.data = self.data.iloc[q].copy()
         new.model_info = self.model_info
@@ -301,25 +285,38 @@ class MESSENGERdata:
         keys.extend([f'data.{col}' for col in self.data.columns])
         return keys
 
+    @staticmethod
+    def _format_query(comparison):
+        """Try to impose some regularity on the query."""
+        query = comparison.lower()
+    
+        # Remove some extra spaces
+        chars = ['=', '>', '<', '!=', '>=', '<=']
+        for char in chars:
+            query = query.replace(f'{char} ', char)
+            query = query.replace(f' {char}', char)
+
+        return query
+
     def set_frame(self, frame=None):
         """Convert between MSO and Model frames.
 
         More frames could be added if necessary.
         If Frame is not specified, flips between MSO and Model."""
-        if (frame is None) and (self.frame.upper() == 'MSO'):
+        if (frame is None) and (self.frame.casefold() == 'MSO'.casefold()):
             frame = 'Model'
-        elif (frame is None) and (self.frame.upper() == 'MODEL'):
+        elif (frame is None) and (self.frame.casefold() == 'MODEL'.casefold()):
             frame = 'MSO'
         else:
             pass
 
-        allframes = ['MODEL', 'MSO']
-        if frame.upper() not in allframes:
+        allframes = ['MODEL'.casefold(), 'MSO'.casefold()]
+        if frame.casefold() not in allframes:
             print('{} is not a valid frame.'.format(frame))
-            return None
         elif frame == self.frame:
             pass
-        elif (self.frame.upper() == 'MSO') and (frame.upper() == 'MODEL'):
+        elif ((self.frame.casefold() == 'MSO'.casefold()) and
+              (frame.casefold() == 'MODEL'.casefold())):
             # Convert from MSO to Model
             self.data.x, self.data.y = self.data.y.copy(), -self.data.x.copy()
             self.data.xbore, self.data.ybore = (self.data.ybore.copy(),
@@ -327,7 +324,8 @@ class MESSENGERdata:
             self.data.xtan, self.data.ytan = (self.data.ytan.copy(),
                                               -self.data.xtan.copy())
             self.frame = 'Model'
-        elif (self.frame.upper() == 'MODEL') and (frame.upper() == 'MODEL'):
+        elif ((self.frame.casefold() == 'MODEL'.casefold()) and
+              (frame.casefold() == 'MODEL'.casefold())):
             self.data.x, self.data.y = -self.data.y.copy(), self.data.x.copy()
             self.data.xbore, self.data.ybore = (-self.data.ybore.copy(),
                                                 self.data.xbore.copy())
@@ -336,11 +334,21 @@ class MESSENGERdata:
             self.frame = 'MSO'
         else:
             assert 0, 'You somehow picked a bad combination.'
+            
 
-    def model(self, start_from, npackets, quantity='radiance',
-              fit_method='chisq', dphi=3*u.deg, overwrite=False,
-              masking=None, filenames=None, label=None,
-              fit_to_data=False, packs_per_it=None):
+    def simulate_determined_source(self, inputs, npackets):
+        pass
+    
+    def model(self,
+              inputs,
+              npackets,
+              quantity='radiance',
+              fit_method='chisq',
+              dphi=1*u.deg,
+              overwrite=False,
+              masking=None,
+              label=None,
+              packs_per_it=None):
         """Run the nexoclom model with specified inputs and fit to the data.
         
         ** Parameters**
@@ -358,7 +366,7 @@ class MESSENGERdata:
             data uncertainty.
             
         dphi
-            Angular size of the view cone. Default = 3 deg. Must be
+            Angular size of the view cone. Default = 1 deg. Must be
             given as an astropy quantity.
             
         overwrite
@@ -376,89 +384,83 @@ class MESSENGERdata:
             
             * siglimitX: Excludes any data where the model is not within X σ
               of the data in an initial fit to the data.
+        
+        start_from_fit
+            Uses a previous fit to the data as the starting point for the model
+            run. Creates a source map based on the fit. Default = False
+            * If start_from_fit = True, the inputs must be the label of the
+              model run that was fit to the data. Must have:
+              data.model_info[inputs]['fitted'] = True
+        
+        start_from_fit_options
+            A dictionary with the fields:
+            * smooth: True if the derived surface map should be smoothed
+              before running. Default = False
+            * nlonbins: Number of longitude bins in grid. Default = 72
+            * nlatbins: Number of latitude bins in grid. Default = 36
+            * nvelbins: Number of velocity bins: Default = 100
+            * mapfile: File to which the derived source should be saved.
+              Should end in '.pkl'. Default='mapfile_temp.pkl'
+            * The speed distribution needs to be defined. It can be defined
+              using the standard SpeedDistribution fields or with
+              type = 'from fit' which approximates the derived speed
+              distribution. Default is 'from fit'
         """
-
-        runmodel = True
-        if isinstance(start_from, str):
-            inputs = Input(start_from)
-            output = None
-        elif isinstance(start_from, Input):
-            inputs = copy.deepcopy(start_from)
-            output = None
-        elif isinstance(start_from, Output):
-            inputs = copy.deepcopy(start_from.inputs)
-            output = start_from
-            runmodel = False
-        else:
-            raise InputError('MESSENGERdata.model', 'Problem with the inputs.')
-
-        # TAA needs to match the data
-        if len(self.data.orbit.unique()) == 1:
-            inputs.geometry.taa = np.median(self.data.taa)*u.rad
-        elif self.data.taa.max()-self.data.taa.min() < 3*np.pi/180.:
-            inputs.geometry.taa = self.data.taa.median()*u.rad
-        else:
-            assert 0, 'Too wide a range of taa'
-            
-        # If using a planet-fixed source map, need to set subsolarlon
-        if ((inputs.spatialdist.type == 'surface map') and
-            (inputs.spatialdist.coordinate_system == 'planet-fixed')):
-            inputs.spatialdist.subsolarlon = self.data.subslong.median()*u.rad
-        else:
-            pass
-
-        # Run the model
+        # Put data into model units
         self.set_frame('Model')
-        if runmodel:
-            inputs.run(npackets, overwrite=overwrite, packs_per_it=packs_per_it)
-            model_result = LOSResult(inputs, self, quantity, dphi=dphi,
-                                     filenames=filenames, overwrite=overwrite,
-                                     masking=masking, fit_to_data=fit_to_data)
+        
+        # Create the LOSResult object
+        model_result = LOSResult(self, quantity, dphi=dphi)
+        
+        # simulate the data
+        if inputs.options.fitted:
+            model_result.determine_source_from_data(inputs, npackets,
+                                                    overwrite=overwrite,
+                                                    packs_per_it=packs_per_it,
+                                                    masking=masking)
         else:
-            model_result = LOSResult(output, self, quantity, dphi=dphi,
-                                     filenames=filenames, overwrite=overwrite,
-                                     fit_method=fit_method, masking=masking,
-                                     fit_to_data=fit_to_data)
-
-        # Simulate the data
-        # modkey is the number for this model
+            model_result.simulate_data_from_inputs(inputs, npackets, overwrite,
+                                                   packs_per_it)
+        
+        # Attach the model_result to the data
         modnum = len(self.model_info)
         modkey = f'model{modnum:00d}'
         npackkey = f'npackets{modnum:00d}'
         maskkey = f'mask{modnum:00d}'
-        self.data[modkey] = model_result.radiance/1e3 # Convert to kR
-        self.data[npackkey] = model_result.ninview
-
-        strength, goodness_of_fit, mask = fit_model(self.data.radiance.values,
-                                                    self.data[modkey].values,
-                                                    self.data.sigma.values,
-                                                    fit_method=fit_method,
-                                                    masking=masking,
-                                                    altitude=self.data.alttan)
-        strength *= u.def_unit('10**23 atoms/s', 1e23/u.s)
         
-        self.data[modkey] = self.data[modkey]*strength.value
+        self.data[modkey] = model_result.radiance / 1e3 # Convert to kR
+        self.data[npackkey] = model_result.npackets
+        strength, goodness_of_fit, mask = mathMB.fit_model(self.data.radiance.values,
+                                                           self.data[modkey].values,
+                                                           self.data.sigma.values,
+                                                           fit_method=fit_method,
+                                                           masking=masking,
+                                                           altitude=self.data.alttan)
+        strength *= u.def_unit('10**23 atoms/s', 1e23 / u.s)
+        self.data[modkey] = self.data[modkey] * strength.value
         self.data[maskkey] = mask
-
+    
         if label is None:
             label = modkey.capitalize()
         else:
             pass
-
-        model_info = {'inputs': inputs,
-                      'fit_method': fit_method,
-                      'goodness-of-fit': goodness_of_fit,
-                      'strength': strength,
-                      'label': label,
-                      'filenames': model_result.filenames,
-                      'fitted': model_result.fitted,
-                      'savefile': model_result.savefiles}
+    
+        model_info = {'inputs':model_result.inputs,
+                      'fit_method':fit_method,
+                      'goodness-of-fit':goodness_of_fit,
+                      'strength':strength,
+                      'label':label,
+                      'outputfiles':model_result.outputfiles,
+                      'fitted':model_result.fitted,
+                      'modelfiles':model_result.modelfiles,
+                      'sourcemap': model_result.sourcemap}
+        
         self.model_info[modkey] = model_info
         print(f'Model strength for {label} = {strength}')
 
     def plot(self, filename=None, plot_method='plotly', show=False):
         if plot_method == 'plotly':
-            app = plot_plotly(self)
+            app = plot_plotly(self, filename)
             if show:
                 app.run_server(debug=False)
             else:
