@@ -4,8 +4,7 @@ import pandas as pd
 import pickle
 from solarsystemMB import SSObject
 from nexoclom import Output
-from astropy.convolution import Gaussian2DKernel
-from astropy.convolution import convolve
+from astropy.convolution import Gaussian2DKernel, convolve
 
 import bokeh.plotting as bkp
 from bokeh.models import (HoverTool, Whisker, CDSView, BooleanFilter,
@@ -51,7 +50,11 @@ def plot_bokeh(self, filename=None, show=True):
     self.data['upper'] = self.data.radiance+self.data.sigma
     self.data['lattandeg'] = self.data.lattan*180/np.pi
     
-    m = self.data[self.data.alttan != self.data.alttan.max()].alttan.max()
+    mask = self.data.alttan != self.data.alttan.max()
+    if np.any(mask):
+        m = self.data[self.data.alttan != self.data.alttan.max()].alttan.max()
+    else:
+        m = 1e10
     col = np.interp(self.data.alttan, np.linspace(0, m, 256),
                     np.arange(256)).astype(int)
     self.data['color'] = [Turbo256[c] for c in col]
@@ -104,19 +107,19 @@ def plot_bokeh(self, filename=None, show=True):
                 c = next(col)
             
             label = f"{info['label']}"
-            fig0.line(x='utc', y=modkey, source=source,
-                      legend_label=label, color=c)
+            # fig0.line(x='utc', y=modkey, source=source,
+            #           legend_label=label, color=c)
             
             maskkey = modkey.replace('model', 'mask')
             mask = (self.data[maskkey]).to_list()
             view = CDSView(source=source, filters=[BooleanFilter(mask)])
-            modplots.append(fig0.circle(x='utc', y=modkey, size=7, color=c,
+            modplots.append(fig0.circle(x='utc', y=modkey, size=9, color=c,
                                         source=source, legend_label=label,
                                         view=view))
             maskkey = modkey.replace('model', 'mask')
             mask = np.logical_not(self.data[maskkey]).to_list()
             view = CDSView(source=source, filters=[BooleanFilter(mask)])
-            maskedplots.append(fig0.circle(x='utc', y=modkey, size=7,
+            maskedplots.append(fig0.circle(x='utc', y=modkey, size=9,
                                            source=source, line_color=c,
                                            fill_color='yellow', view=view,
                                            legend_label=label + ' (Data Point Not Used)'))
@@ -169,7 +172,7 @@ def plot_bokeh(self, filename=None, show=True):
     return fig0, fig1
 
 
-def plot_plotly(self, show=True):
+def plot_plotly(self, filename=None):
     pio.templates.default = 'plotly_white'
     
     # Format the date correction
@@ -189,7 +192,8 @@ def plot_plotly(self, show=True):
                               hovermode='x',
                               clickmode='event+select',
                               )
-    
+    data_figure.update_xaxes(rangeslider_visible=True)
+
     datafig = go.Scatter(name='Data',
                          x=self.data.utc,
                          y=self.data.radiance,
@@ -233,7 +237,9 @@ def plot_plotly(self, show=True):
                           line={'color':'purple'})
     data_figure.add_trace(dataline)
     data_figure.update_yaxes(range=[0, dmax*1.2])
-    
+    if filename is not None:
+        data_figure.write_html(filename)
+
     ##########################
     # Make the tangent point plot
     amax = self.data[self.data.alttan != self.data.alttan.max()].alttan.max()
@@ -453,7 +459,7 @@ def plot_plotly(self, show=True):
     return app
 
 
-# def get_radiance_source(self, nlonbins=72, nlatbins=36, nvelbins=100):
+# def make_final_source(self, nlonbins=72, nlatbins=36, nvelbins=100):
 #     modkey = None
 #     for key in self.model_info:
 #         if self.model_info[key]['fitted']:
@@ -464,33 +470,31 @@ def plot_plotly(self, show=True):
 #     '''Make source frames for each spectrum'''
 #     Mercury = SSObject('Mercury')
 #     model_info = self.model_info[modkey]
-#     loctime, latitude = np.ndarray((0,)), np.ndarray((0,))
-#     velocity, weight = np.ndarray((0,)), np.ndarray((0,))
+#     loctime, latitude = np.zeros((0,)), np.zeros((0,))
+#     velocity, weight = np.zeros((0,)), np.zeros((0,))
 #
 #     # Load the data from all available outputfiles
 #     for outputfile, modelfile in model_info['savefile'].items():
 #         output = Output.restore(outputfile)
 #         with open(modelfile, 'rb') as f:
 #             model_result = pickle.load(f)
-#         assert len(model_result) == 4, 'Not a proper fitted model result.'
-#
-#         _, _, weighting, packets = model_result
 #
 #         # Final source information
-#         new_weight = weighting.weight.apply(
+#         new_weight = model_result['weighting'].weight.apply(
 #                 lambda x:x.mean() if x.shape[0] > 0 else 0.)
+#         include = model_result['weighting'].weight.apply(lambda x:len(x) > 0)
 #         if np.any(new_weight > 0):
 #             multiplier = new_weight.loc[output.X['Index']].values
 #             output.X.loc[:, 'frac'] = output.X.loc[:, 'frac']*multiplier
 #             output.X0.loc[:, 'frac'] = output.X0.loc[:, 'frac']*new_weight
 #             output.totalsource = np.sum(output.X0.frac)
 #
-#             loctime = np.append(loctime, output.X0.local_time)
-#             latitude = np.append(latitude, output.X0.latitude)
-#             velocity = np.append(velocity, np.sqrt(output.X0.vx**2 +
-#                                                    output.X0.vy**2 +
-#                                                    output.X0.vz**2))
-#             weight = np.append(weight, output.X0.frac)
+#             loctime = np.append(loctime, output.X0.loc[include].local_time)
+#             latitude = np.append(latitude, output.X0.loc[include].latitude)
+#             velocity = np.append(velocity, np.sqrt(output.X0.loc[include].vx**2 +
+#                                                    output.X0.loc[include].vy**2 +
+#                                                    output.X0.loc[include].vz**2))
+#             weight = np.append(weight, output.X0.loc[include].frac)
 #         else:
 #             pass
 #
@@ -503,8 +507,7 @@ def plot_plotly(self, show=True):
 #         source, xx, yy = np.histogram2d(loctime, latitude, weights=weight,
 #                                         range=[[0, 24], [-90, 90]],
 #                                         bins=(nlonbins, nlatbins))
-#         v_source, v = np.histogram(velocity,
-#                                    bins=np.arange(0, np.max(velocity), .1),
+#         v_source, v = np.histogram(velocity, bins=nvelbins, range=[0, 10],
 #                                    weights=weight)
 #         v_source /= np.max(v_source)
 #
@@ -512,117 +515,28 @@ def plot_plotly(self, show=True):
 #         packets, _, _ = np.histogram2d(loctime, latitude,
 #                                        range=[[0, 24], [-90, 90]],
 #                                        bins=(nlonbins, nlatbins))
-#         v_packets, _ = np.histogram(velocity,
-#                                     bins=np.arange(0, np.max(velocity), .1))
+#         v_packets, _ = np.histogram(velocity, bins=nvelbins, range=[0, 10])
 #         v_packets = v_packets/np.max(v_packets)
 #
 #         # Make local_time, latitude, velocity the center of the bins
-#         local_time = xx+(xx[1]-xx[0])/2.
-#         latitude = yy+(yy[1]-yy[0])/2.
-#         v = v+(v[1]-v[0])/2.
+#         local_time = xx[:-1] + (xx[1]-xx[0])/2.
+#         latitude = yy[:-1] + (yy[1]-yy[0])/2.
+#         v = v[:-1] + (v[1]-v[0])/2.
 #
-#         assert 0
-#         source = source/np.cos(latitude[:, np.newaxis]*np.pi/180)
-#         packets = packets/np.cos(latitude[:, np.newaxis]*np.pi/180)
+#         # Correct for surface area
+#         lat = np.cos(latitude*np.pi/180)
+#         source = source/lat[np.newaxis,:]
+#         packets = packets/lat[np.newaxis,:]
 #
 #         result = {'source':source, 'packets':packets,
-#                   'local_time':local_time[:-1], 'latitude':latitude[:-1],
-#                   'v_source':v_source, 'v_packets':v_packets, 'v':v[:-1]}
+#                   'local_time':local_time, 'latitude':latitude,
+#                   'v_source':v_source, 'v_packets':v_packets, 'velocity':v}
 #     else:
-#         local_time = np.linspace(0, 24, nlonbins, endpoint=False)
-#         local_time += (local_time[1]-local_time[0])/2.
-#         latitude = np.linspace(-90, 92, nlatbins, endpoint=False)
-#         latitude += (local_time[1]-local_time[0])/2.
-#         result = {'source':np.zeros((nlatbins, nlonbins)),
-#                   'packets':np.zeros((nlatbins, nlonbins)),
-#                   'local_time':local_time,
-#                   'latitude':latitude,
-#                   'v_source':np.zeros(10), 'v_packets':np.zeros(10),
-#                   'v':np.linspace(0, 1, 10)}
+#         result = None
+#
 #     return result
 
 
-def make_final_source(self, nlonbins=72, nlatbins=36, nvelbins=100):
-    modkey = None
-    for key in self.model_info:
-        if self.model_info[key]['fitted']:
-            modkey = key
-        else:
-            pass
-    
-    '''Make source frames for each spectrum'''
-    Mercury = SSObject('Mercury')
-    model_info = self.model_info[modkey]
-    loctime, latitude = np.zeros((0,)), np.zeros((0,))
-    velocity, weight = np.zeros((0,)), np.zeros((0,))
-    
-    # Load the data from all available outputfiles
-    for outputfile, modelfile in model_info['savefile'].items():
-        output = Output.restore(outputfile)
-        with open(modelfile, 'rb') as f:
-            model_result = pickle.load(f)
-        assert len(model_result) == 4, 'Not a proper fitted model result.'
-        
-        _, _, weighting, packets = model_result
-        
-        # Final source information
-        new_weight = weighting.weight.apply(
-                lambda x:x.mean() if x.shape[0] > 0 else 0.)
-        include = weighting.weight.apply(lambda x:len(x) > 0)
-        if np.any(new_weight > 0):
-            multiplier = new_weight.loc[output.X['Index']].values
-            output.X.loc[:, 'frac'] = output.X.loc[:, 'frac']*multiplier
-            output.X0.loc[:, 'frac'] = output.X0.loc[:, 'frac']*new_weight
-            output.totalsource = np.sum(output.X0.frac)
-            
-            loctime = np.append(loctime, output.X0.loc[include].local_time)
-            latitude = np.append(latitude, output.X0.loc[include].latitude)
-            velocity = np.append(velocity, np.sqrt(output.X0.loc[include].vx**2 +
-                                                   output.X0.loc[include].vy**2 +
-                                                   output.X0.loc[include].vz**2))
-            weight = np.append(weight, output.X0.loc[include].frac)
-        else:
-            pass
-
-    # Produce the necessary histograms
-    if len(loctime) > 0:
-        velocity *= Mercury.radius.value
-        latitude *= 180./np.pi
-        
-        # source used
-        source, xx, yy = np.histogram2d(loctime, latitude, weights=weight,
-                                        range=[[0, 24], [-90, 90]],
-                                        bins=(nlonbins, nlatbins))
-        v_source, v = np.histogram(velocity, bins=nvelbins, range=[0, 10],
-                                   weights=weight)
-        v_source /= np.max(v_source)
-        
-        # packets available
-        packets, _, _ = np.histogram2d(loctime, latitude,
-                                       range=[[0, 24], [-90, 90]],
-                                       bins=(nlonbins, nlatbins))
-        v_packets, _ = np.histogram(velocity, bins=nvelbins, range=[0, 10])
-        v_packets = v_packets/np.max(v_packets)
-
-        # Make local_time, latitude, velocity the center of the bins
-        local_time = xx[:-1] + (xx[1]-xx[0])/2.
-        latitude = yy[:-1] + (yy[1]-yy[0])/2.
-        v = v[:-1] + (v[1]-v[0])/2.
-
-        # Correct for surface area
-        lat = np.cos(latitude*np.pi/180)
-        source = source/lat[np.newaxis,:]
-        packets = packets/lat[np.newaxis,:]
-
-        result = {'source':source, 'packets':packets,
-                  'local_time':local_time, 'latitude':latitude,
-                  'v_source':v_source, 'v_packets':v_packets, 'velocity':v}
-    else:
-        result = None
-    
-    return result
-
-
 def frame_generator(self, nlonbins=72, nlatbins=36, nvelbins=100):
     '''Make source frames for each spectrum'''
     modkey = None
@@ -645,13 +559,12 @@ def frame_generator(self, nlonbins=72, nlatbins=36, nvelbins=100):
         output = Output.restore(outputfile)
         with open(modelfile, 'rb') as f:
             model_result = pickle.load(f)
-        assert len(model_result) == 4, 'Not a proper fitted model result.'
         
         alloutputs.append(output)
-        new_weight = model_result[2].weight.apply(
+        new_weight = model_result['weighting'].weight.apply(
                 lambda x:x.mean() if x.shape[0] > 0 else 0.)
         allweights.append(new_weight)
-        allpackets.append(model_result[3])
+        allpackets.append(model_result['packets'])
     
     for specnum in self.data.index:
         loctime, latitude = np.zeros((0,)), np.zeros((0,))
@@ -714,15 +627,19 @@ def make_fitted_plot(self, result, filestart='fitted', show=True, ut=None,
     
     if smooth:
         kernel = Gaussian2DKernel(x_stddev=1)
-        source = convolve(result['source'], kernel)
-        packets = convolve(result['packets'], kernel)
+        source = convolve(result['abundance'], kernel)
+        packets = convolve(result['p_available'], kernel)
     else:
-        source = result['source']
-        packets = result['packets']
+        source = result['abundance']
+        packets = result['p_available']
     
     # Tools
     tools = ['save']
-    
+
+    local_time = (result['longitude'].value * 12/np.pi + 12) % 24
+    arg = np.argsort(local_time[:-1])
+    source, packets = source[arg,:], packets[arg,:]
+
     # Distribution of available packets
     fig0 = bkp.figure(plot_width=WIDTH, plot_height=HEIGHT,
                       title=f'{self.species}, {self.query}, Available Packets',
@@ -771,9 +688,9 @@ def make_fitted_plot(self, result, filestart='fitted', show=True, ut=None,
     fig2.xaxis.major_label_text_font_size = NUMFONTSIZE
     fig2.yaxis.major_label_text_font_size = NUMFONTSIZE
     
-    fig2.line(x=result['velocity'], y=result['v_packets'],
+    fig2.line(x=result['velocity'][:-1], y=result['v_available'],
               legend_label='Packets Available', color='red')
-    fig2.line(x=result['velocity'], y=result['v_source'],
+    fig2.line(x=result['velocity'][:-1], y=result['vdist'],
               legend_label='Packets Used', color='blue')
     
     # Full orbit time series
@@ -883,7 +800,7 @@ def make_fitted_plot(self, result, filestart='fitted', show=True, ut=None,
     grid = gridplot([[fig3, fig2], [fig0, fig1]])
     
     # Save png version
-    export_png(grid, filename=filestart+'.png')
+    # export_png(grid, filename=filestart+'.png')
     
     bkp.output_file(filestart+'.html')
     bkp.save(grid)  # html files not needed
@@ -903,24 +820,31 @@ def plot_fitted(self, filestart=None, show=True, make_frames=False, smooth=False
         os.makedirs(os.path.dirname(filestart))
     
     # Compute and plot final source results
-    final_result = make_final_source(self)
-    make_fitted_plot(self, final_result, filestart=filestart, show=show,
-                     smooth=smooth)
+    for key in self.model_info:
+        if self.model_info[key]['fitted']:
+            final_result = self.model_info[key]['sourcemap']
+            make_fitted_plot(self, final_result, filestart=filestart, show=show,
+                             smooth=smooth)
     
-    if make_frames:
-        # Compute results for each spectrum
-        frames_result = frame_generator(self)
-        allframes = frames_result['result']
-        for specnum, frame in allframes.iterrows():
-            print(specnum)
-            result = frame.to_dict()
-            result['velocity'] = final_result['velocity']
-            
-            framefilestart = f'{filestart}_{specnum}'
-            make_fitted_plot(self, result, framefilestart, False,
-                             ut=self.data.loc[specnum, 'utc'], smooth=smooth)
+            if make_frames:
+                # Compute results for each spectrum
+                frames_result = frame_generator(self)
+                allframes = frames_result['result']
+                for specnum, frame in allframes.iterrows():
+                    print(specnum)
+                    result = frame.to_dict()
+                    result['velocity'] = final_result['velocity']
+                    
+                    framefilestart = f'{filestart}_{specnum}'
+                    make_fitted_plot(self, result, framefilestart, False,
+                                     ut=self.data.loc[specnum, 'utc'], smooth=smooth)
         
-        # Animate the frames
-        os.system(f'convert -delay 10 -quality 100 {filestart}*.png {filestart}.mpeg')
-        os.system(f'rm {filestart}_*.png')
-        os.system(f'rm {filestart}_*.html')
+                    # Animate the frames
+                    os.system(f'convert -delay 10 -quality 100 {filestart}*.png '
+                              f'{filestart}.mpeg')
+                    os.system(f'rm {filestart}_*.png')
+                    os.system(f'rm {filestart}_*.html')
+            else:
+                pass
+        else:
+            pass
